@@ -1,21 +1,17 @@
 package HTML::Template::Compiled::Expression;
-# $Id: Expression.pm 665 2006-10-02 15:48:02Z tinita $
+# $Id: Expression.pm 1140 2011-11-28 00:25:38Z tinita $
 use strict;
 use warnings;
 
 use constant OPERANDS => 0;
 use constant ATTRIBUTES => 1;
 use base 'Exporter';
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 use  HTML::Template::Compiled::Expression::Expressions;
 my @expressions = qw(
-    &_expr_close
-    &_expr_open
-    &_expr_else
     &_expr_literal
     &_expr_defined
     &_expr_ternary
-    &_expr_string
     &_expr_function
     &_expr_method
     &_expr_elsif
@@ -25,13 +21,9 @@ our %EXPORT_TAGS = (
     expressions => \@expressions,
 );
 
-sub _expr_close { HTML::Template::Compiled::Expression::Close->new }
-sub _expr_open { HTML::Template::Compiled::Expression::Open->new }
-sub _expr_else { HTML::Template::Compiled::Expression::Else->new }
 sub _expr_literal { HTML::Template::Compiled::Expression::Literal->new(@_) }
 sub _expr_defined { HTML::Template::Compiled::Expression::Defined->new(@_) }
 sub _expr_ternary { HTML::Template::Compiled::Expression::Ternary->new(@_) }
-sub _expr_string { HTML::Template::Compiled::Expression::String->new(@_) }
 sub _expr_function { HTML::Template::Compiled::Expression::Function->new(@_) }
 sub _expr_method { HTML::Template::Compiled::Expression::Method->new(@_) }
 sub _expr_elsif { HTML::Template::Compiled::Expression::Elsif->new(@_) }
@@ -67,6 +59,7 @@ sub to_string { print "$_[0] to_string\n" }
 
 sub level2indent {
     my ($self, $level) = @_;
+    no warnings;
     return "  " x $level;
 }
 
@@ -79,26 +72,6 @@ sub init {
     $self->set_operands([$op]);
 }
 
-package HTML::Template::Compiled::Expression::If;
-our @ISA = qw(HTML::Template::Compiled::Expression::Conditional);
-
-sub to_string {
-    my ($self, $level) = @_;
-    my $indent = $self->level2indent($level);
-    my ($op) = $self->get_operands;
-    return $indent . 'if ( ' . $op->to_string . ' ) {';
-}
-
-package HTML::Template::Compiled::Expression::Unless;
-our @ISA = qw(HTML::Template::Compiled::Expression::Conditional);
-
-sub to_string {
-    my ($self, $level) = @_;
-    my $indent = $self->level2indent($level);
-    my ($op) = $self->get_operands;
-    return $indent . 'unless ( ' . $op->to_string . ' ) {';
-}
-
 package HTML::Template::Compiled::Expression::Elsif;
 our @ISA = qw(HTML::Template::Compiled::Expression::Conditional);
 
@@ -109,15 +82,6 @@ sub to_string {
     return $indent . '}' . $/ . $indent . 'elsif ( ' . $op->to_string . ' ) {';
 }
 
-package HTML::Template::Compiled::Expression::Else;
-our @ISA = qw(HTML::Template::Compiled::Expression::Conditional);
-
-sub to_string {
-    my ($self, $level) = @_;
-    my $indent = $self->level2indent($level);
-    return $indent . '}' . $/ . $indent . 'else {';
-}
-
 package HTML::Template::Compiled::Expression::Function;
 our @ISA = qw(HTML::Template::Compiled::Expression);
 
@@ -129,10 +93,10 @@ sub to_string {
     my ($self, $level) = @_;
     my $indent = $self->level2indent($level);
     my ($function, @ops) = $self->get_operands;
-    return "$indent$function( " .
-        join(", ", map {
-                $_->to_string($level)
-        } @ops) . " )";
+    my @strings = map {
+        ref $_ ? $_->to_string($level) : $_
+    } @ops;
+    return "$indent$function( " . (join ',', @strings). ')';
 }
 
 package HTML::Template::Compiled::Expression::Method;
@@ -142,11 +106,12 @@ sub to_string {
     my ($self, $level) = @_;
     my $indent = $self->level2indent($level);
     my ($function, $object, @args) = $self->get_operands;
-    return $indent . $object->to_string($level) .
-        "->$function( " .
-        join(", ", map {
-                $_->to_string($level)
-        } @args) . " )";
+    my $start = $indent . (ref $object ? $object->to_string($level) : $object)
+        . "->$function( ";
+    my @strings = map {
+        ref $_ ? $_->to_string($level) : $_
+    } @args;
+    return $start . (join ',', @strings). ')';
 }
 
 1;
@@ -156,7 +121,7 @@ __END__
 
 =head1 NAME
 
-HTML::Template::Compiled::Expression
+HTML::Template::Compiled::Expression - a compiled HTML template expression
 
 =head1 DESCRIPTION
 
